searchData={"items":[{"type":"module","title":"PhoenixTest.Playwright","doc":"Run feature tests in an actual browser, using [PhoenixTest](https://hexdocs.pm/phoenix_test) and [Playwright](https://playwright.dev/).\n\n```elixir\ndefmodule Features.RegisterTest do\n  use PhoenixTest.Playwright.Case,\n    async: true,\n    parameterize: [                      # run in multiple browsers in parallel\n      %{browser: :chromium},\n      %{browser: :firefox}\n    ],\n    headless: false,                     # show browser window\n    slow_mo: :timer.seconds(1)           # add delay between interactions\n\n  @tag trace: :open                      # replay in interactive viewer\n  test \"register\", %{conn: conn} do\n    conn\n    |> visit(~p\"/\")\n    |> click_link(\"Register\")\n\n    |> fill_in(\"Email\", with: \"f@ftes.de\")\n    |> click_button(\"Create an account\")\n\n    |> assert_has(\".text-rose-600\", text: \"required\")\n    |> screenshot(\"error.png\", full_page: true)\n  end\nend\n```\n\nPlease [get in touch](https://ftes.de) with feedback of any shape and size.\n\nEnjoy! Freddy.","ref":"PhoenixTest.Playwright.html"},{"type":"module","title":"Getting started - PhoenixTest.Playwright","doc":"1. Add dependency\n  ```elixir\n  # mix.exs\n  {:phoenix_test_playwright, \"~> 0.4\", only: :test, runtime: false}\n  ```\n\n2. Install playwright and browser\n  ```\n  npm --prefix assets i -D playwright\n  npm --prefix assets exec playwright install chromium --with-deps\n  ```\n\n3. Config\n  ```elixir\n  # config/test.exs\n  config :phoenix_test, otp_app: :your_app\n  config :your_app, YourAppWeb.Endpoint, server: true\n  ```\n\n4. Runtime config\n  ```elixir\n  # test/test_helpers.exs\n  Application.put_env(:phoenix_test, :base_url, YourAppWeb.Endpoint.url()\n  ```\n\n5. Use in test\n  ```elixir\n  defmodule MyTest do\n    use PhoenixTest.Playwright.Case, async: true\n\n    # `conn` isn't a `Plug.Conn` but a Playwright session.\n    # We use the name `conn` anyway so you can easily switch `PhoenixTest` drivers.\n    test \"in browser\", %{conn: conn} do\n      conn\n      |> visit(~p\"/\")\n      |> unwrap(&Frame.evaluate(&1.frame_id, \"console.log('Hey')\"))\n  ```\n\n> #### Reference project {: .neutral}\n>\n> [github.com/ftes/phoenix_test_playwright_example](https://github.com/ftes/phoenix_test_playwright_example)\n>\n> The last commit adds a feature test for the `phx gen.auth` registration page\n> and runs it in CI (Github Actions).","ref":"PhoenixTest.Playwright.html#module-getting-started"},{"type":"module","title":"Configuration - PhoenixTest.Playwright","doc":"```elixir\n# config/test.ex\nconfig :phoenix_test,\n  otp_app: :your_app,\n  playwright: [\n    browser: :chromium,\n    headless: System.get_env(\"PW_HEADLESS\", \"true\") in ~w(t true),\n    js_logger: false,\n    screenshot: System.get_env(\"PW_SCREENSHOT\", \"false\") in ~w(t true),\n    trace: System.get_env(\"PW_TRACE\", \"false\") in ~w(t true),\n    browser_launch_timeout: 10_000,\n  ]\n```\n\nSee `PhoenixTest.Playwright.Config` for more details.\n\nYou can override some options in your test:\n\n```elixir\ndefmodule DebuggingFeatureTest do\n  use PhoenixTest.Playwright.Case,\n    async: true,\n    # Show browser and pause 1 second between every interaction\n    headless: false,\n    slow_mo: :timer.seconds(1)\n```","ref":"PhoenixTest.Playwright.html#module-configuration"},{"type":"module","title":"Traces - PhoenixTest.Playwright","doc":"Playwright traces record a full browser history, including 'user' interaction, browser console, network transfers etc.\nTraces can be explored in an interactive viewer for debugging purposes.","ref":"PhoenixTest.Playwright.html#module-traces"},{"type":"module","title":"Manually - PhoenixTest.Playwright","doc":"```elixir\n@tag trace: :open\ntest \"record a trace and open it automatically in the viewer\" do\n```","ref":"PhoenixTest.Playwright.html#module-manually"},{"type":"module","title":"Automatically for failed tests in CI - PhoenixTest.Playwright","doc":"```elixir\n# config/test.exs\nconfig :phoenix_test, playwright: [trace: System.get_env(\"PW_TRACE\", \"false\") in ~w(t true)]\n```\n\n```yaml\n# .github/workflows/elixir.yml\nrun: \"mix test || if [[ $? = 2 ]]; then PW_TRACE=true mix test --failed; else false; fi\"\n```","ref":"PhoenixTest.Playwright.html#module-automatically-for-failed-tests-in-ci"},{"type":"module","title":"Screenshots - PhoenixTest.Playwright","doc":"","ref":"PhoenixTest.Playwright.html#module-screenshots"},{"type":"module","title":"Manually - PhoenixTest.Playwright","doc":"```elixir\n|> visit(~p\"/\")\n|> screenshot(\"home.png\")    # captures entire page by default, not just viewport\n```","ref":"PhoenixTest.Playwright.html#module-manually"},{"type":"module","title":"Automatically for failed tests in CI - PhoenixTest.Playwright","doc":"```elixir\n# config/test.exs\nconfig :phoenix_test, playwright: [screenshot: System.get_env(\"PW_SCREENSHOT\", \"false\") in ~w(t true)]\n```\n\n```yaml\n# .github/workflows/elixir.yml\nrun: \"mix test || if [[ $? = 2 ]]; then PW_SCREENSHOT=true mix test --failed; else false; fi\"\n```","ref":"PhoenixTest.Playwright.html#module-automatically-for-failed-tests-in-ci"},{"type":"module","title":"Emails - PhoenixTest.Playwright","doc":"If you want to verify the HTML of sent emails in your feature tests,\nconsider using `Plug.Swoosh.MailboxPreview`.\nThe iframe used to render the email HTML body makes this slightly tricky:\n\n```elixir\n|> visit(~p\"/dev/mailbox\")\n|> click_link(\"Confirmation instructions\")\n|> within(\"iframe >> internal:control=enter-frame\", fn conn ->\n  conn\n  |> click_link(\"Confirm account\")\n  |> click_button(\"Confirm my account\")\n  |> assert_has(\"#flash-info\", text: \"User confirmed\")\nend)\n```\n\nFor a full example see [ftes/phoenix_test_playwright_example/tree/phoenix-1.8](https://github.com/ftes/phoenix_test_playwright_example/tree/phoenix-1.8).","ref":"PhoenixTest.Playwright.html#module-emails"},{"type":"module","title":"Common problems - PhoenixTest.Playwright","doc":"","ref":"PhoenixTest.Playwright.html#module-common-problems"},{"type":"module","title":"Test failure in CI (timeout) - PhoenixTest.Playwright","doc":"- Limit concurrency: `mix test --max-cases 1` for GitHub CI shared runners\n- Increase timemout: `config :phoenix_test, playwright: [timeout: :timer.seconds(4)]`\n- More compute power: e.g. `x64 8-core` [GitHub runner](https://docs.github.com/en/enterprise-cloud@latest/actions/using-github-hosted-runners/using-larger-runners/about-larger-runners#machine-sizes-for-larger-runners)","ref":"PhoenixTest.Playwright.html#module-test-failure-in-ci-timeout"},{"type":"module","title":"LiveView not connected - PhoenixTest.Playwright","doc":"```elixir\n|> visit(~p\"/\")\n|> assert_has(\"body .phx-connected\")\n# now continue, playwright has waited for LiveView to connect\n```","ref":"PhoenixTest.Playwright.html#module-liveview-not-connected"},{"type":"module","title":"LiveComponent not connected - PhoenixTest.Playwright","doc":"```html\n \n```\n\n```elixir\n|> visit(~p\"/\")\n|> assert_has(\"#my-component[data-connected]\")\n# now continue, playwright has waited for LiveComponent to connect\n```","ref":"PhoenixTest.Playwright.html#module-livecomponent-not-connected"},{"type":"module","title":"Ecto SQL.Sandbox - PhoenixTest.Playwright","doc":"```elixir\ndefmodule MyTest do\n  use PhoenixTest.Playwright.Case, async: true\n```\n\n`PhoenixTest.Playwright.Case` automatically takes care of this.\nIt passes a user agent referencing your Ecto repos.\nThis allows for [concurrent browser tests](https://hexdocs.pm/phoenix_ecto/main.html#concurrent-browser-tests).\n\nMake sure to follow the advanced set up instructions if necessary:\n- [with LiveViews](https://hexdocs.pm/phoenix_ecto/Phoenix.Ecto.SQL.Sandbox.html#module-acceptance-tests-with-liveviews)\n- [with Channels](https://hexdocs.pm/phoenix_ecto/Phoenix.Ecto.SQL.Sandbox.html#module-acceptance-tests-with-channels)","ref":"PhoenixTest.Playwright.html#module-ecto-sql-sandbox"},{"type":"module","title":"Missing Playwright features - PhoenixTest.Playwright","doc":"This module includes functions that are not part of the PhoenixTest protocol, e.g. `screenshot/3` and `click_link/4`.\n\nBut it does not wrap the entire Playwright API, which is quite large.\nYou should be able to add any missing functionality yourself\nusing `PhoenixTest.unwrap/2`, [`Frame`](`PhoenixTest.Playwright.Frame`), [`Selector`](`PhoenixTest.Playwright.Selector`),\nand the [Playwright code](https://github.com/microsoft/playwright/blob/main/packages/playwright-core/src/client/frame.ts).\n\nIf you think others might benefit, please [open a PR](https://github.com/ftes/phoenix_test_playwright/pulls).\n\nHere is some inspiration:\n\n```elixir\ndefp assert_a11y(session) do\n  Frame.evaluate(session.frame_id, A11yAudit.JS.axe_core())\n\n  results =\n    session.frame_id\n    |> Frame.evaluate(\"axe.run()\")\n    |> A11yAudit.Results.from_json()\n\n  A11yAudit.Assertions.assert_no_violations(results)\n\n  session\nend\n\ndef assert_download(session, name, contains: content) do\n  assert_receive({:playwright, %{method: :download} = download_msg}, 2000)\n  artifact_guid = download_msg.params.artifact.guid\n  assert_receive({:playwright, %{method: :__create__, params: %{guid: ^artifact_guid}} = artifact_msg}, 2000)\n  download_path = artifact_msg.params.initializer.absolute_path\n  wait_for_file(download_path)\n\n  assert download_msg.params.suggested_filename =~ name\n  assert File.read!(download_path) =~ content\n\n  session\nend\n\ndef assert_has_value(session, label, value, opts \\\\ []) do\n  opts = Keyword.validate!(opts, exact: true)\n\n  assert_found(session,\n    selector: Selector.label(label, opts),\n    expression: \"to.have.value\",\n    expectedText: [%{string: value}]\n  )\nend\n\ndef assert_has_selected(session, label, value, opts \\\\ []) do\n  opts = Keyword.validate!(opts, exact: true)\n\n  assert_found(session,\n    selector: label |> Selector.label(opts) |> Selector.concat(\"option[selected]\"),\n    expression: \"to.have.text\",\n    expectedText: [%{string: value}]\n  )\nend\n\ndef assert_is_chosen(session, label, opts \\\\ []) do\n  opts = Keyword.validate!(opts, exact: true)\n\n  assert_found(session,\n    selector: Selector.label(label, opts),\n    expression: \"to.have.attribute\",\n    expressionArg: \"checked\"\n  )\nend\n\ndef assert_is_editable(session, label, opts \\\\ []) do\n  opts = Keyword.validate!(opts, exact: true)\n\n  assert_found(session,\n    selector: Selector.label(label, opts),\n    expression: \"to.be.editable\"\n  )\nend\n\ndef refute_is_editable(session, label, opts \\\\ []) do\n  opts = Keyword.validate!(opts, exact: true)\n\n  assert_found(\n    session,\n    [\n      selector: Selector.label(label, opts),\n      expression: \"to.be.editable\"\n    ],\n    is_not: true\n  )\nend\n\ndef assert_found(session, params, opts \\\\ []) do\n  is_not = Keyword.get(opts, :is_not, false)\n  params = Enum.into(params, %{is_not: is_not})\n\n  unwrap(session, fn frame_id ->\n    {:ok, found} = Frame.expect(frame_id, params)\n    if is_not, do: refute(found), else: assert(found)\n  end)\nend\n\ndefp wait_for_file(path, remaining_ms \\\\ 2000, wait_for_ms \\\\ 100)\ndefp wait_for_file(path, remaining_ms, _) when remaining_ms <= 0, do: flunk(\"File #{path} does not exist\")\n\ndefp wait_for_file(path, remaining_ms, wait_for_ms) do\n  if File.exists?(path) do\n    :ok\n  else\n    Process.sleep(wait_for_ms)\n    wait_for_file(path, remaining_ms - wait_for_ms, wait_for_ms)\n  end\nend\n\ndefp within_iframe(selector \\\\ \"iframe\", fun) when is_function(fun, 1) do\n  within(\"#{selector} >> internal:control=enter-frame\", fun)\nend\n```","ref":"PhoenixTest.Playwright.html#module-missing-playwright-features"},{"type":"function","title":"PhoenixTest.Playwright.add_cookies/2","doc":"Add cookies to the browser context, using `Plug.Conn.put_resp_cookie/3`\n\nNote that for signed cookies the signing salt is **not** configurable.\nAs such, this function is not appropriate for signed `Plug.Session` cookies.\nFor signed session cookies, use `add_session_cookie/3`\n\nSee `PhoenixTest.Playwright.CookieArgs` for the type of the cookie.","ref":"PhoenixTest.Playwright.html#add_cookies/2"},{"type":"function","title":"PhoenixTest.Playwright.add_session_cookie/3","doc":"Add a `Plug.Session` cookie to the browser context.\n\nThis is useful for emulating a logged-in user.\n\nNote that that the cookie `:value` must be a map, since we are using\n`Plug.Conn.put_session/3` to write each of value's key-value pairs\nto the cookie.\n\nThe `session_options` are exactly the same as the opts used when\nwriting `plug Plug.Session` in your router/endpoint module.","ref":"PhoenixTest.Playwright.html#add_session_cookie/3"},{"type":"function","title":"Examples - PhoenixTest.Playwright.add_session_cookie/3","doc":"|> add_session_cookie(\n      [value: %{user_token: Accounts.generate_user_session_token(user)}],\n      MyAppWeb.Endpoint.session_options()\n    )","ref":"PhoenixTest.Playwright.html#add_session_cookie/3-examples"},{"type":"function","title":"PhoenixTest.Playwright.clear_cookies/2","doc":"Removes all cookies from the context","ref":"PhoenixTest.Playwright.html#clear_cookies/2"},{"type":"function","title":"PhoenixTest.Playwright.click/2","doc":"See `click/4`.","ref":"PhoenixTest.Playwright.html#click/2"},{"type":"function","title":"PhoenixTest.Playwright.click/4","doc":"Click an element that is not a link or button.\nOtherwise, use `click_link/4` and `click_button/4`.","ref":"PhoenixTest.Playwright.html#click/4"},{"type":"function","title":"Options - PhoenixTest.Playwright.click/4","doc":"* `:exact` (`t:boolean/0`) - Exact or substring text match. The default value is `false`.","ref":"PhoenixTest.Playwright.html#click/4-options"},{"type":"function","title":"Examples - PhoenixTest.Playwright.click/4","doc":"|> click(Selector.menuitem(\"Edit\", exact: true))\n    |> click(\"summary\", \"(expand)\", exact: false)","ref":"PhoenixTest.Playwright.html#click/4-examples"},{"type":"function","title":"PhoenixTest.Playwright.click_button/4","doc":"Like `PhoenixTest.click_button/3`, but allows exact text match.","ref":"PhoenixTest.Playwright.html#click_button/4"},{"type":"function","title":"Options - PhoenixTest.Playwright.click_button/4","doc":"* `:exact` (`t:boolean/0`) - Exact or substring text match. The default value is `false`.","ref":"PhoenixTest.Playwright.html#click_button/4-options"},{"type":"function","title":"PhoenixTest.Playwright.click_link/4","doc":"Like `PhoenixTest.click_link/3`, but allows exact text match.","ref":"PhoenixTest.Playwright.html#click_link/4"},{"type":"function","title":"Options - PhoenixTest.Playwright.click_link/4","doc":"* `:exact` (`t:boolean/0`) - Exact or substring text match. The default value is `false`.","ref":"PhoenixTest.Playwright.html#click_link/4-options"},{"type":"type","title":"PhoenixTest.Playwright.css_selector/0","doc":"","ref":"PhoenixTest.Playwright.html#t:css_selector/0"},{"type":"type","title":"PhoenixTest.Playwright.playwright_selector/0","doc":"","ref":"PhoenixTest.Playwright.html#t:playwright_selector/0"},{"type":"function","title":"PhoenixTest.Playwright.press/4","doc":"Focuses the matching element and presses a combination of the keyboard keys.\n\nUse `type/4` if you don't need to press special keys.\n\nExamples of [supported keys](https://developer.mozilla.org/en-US/docs/Web/API/UI_Events/Keyboard_event_key_values):\n`F1 - F12, Digit0- Digit9, KeyA- KeyZ, Backquote, Minus, Equal, Backslash, Backspace, Tab, Delete, Escape, ArrowDown, End, Enter, Home, Insert, PageDown, PageUp, ArrowRight, ArrowUp`\n\nModifiers are also supported:\n`Shift, Control, Alt, Meta, ShiftLeft, ControlOrMeta`\n\nCombinations are also supported:\n`Control+o, Control++, Control+Shift+T`","ref":"PhoenixTest.Playwright.html#press/4"},{"type":"function","title":"Options - PhoenixTest.Playwright.press/4","doc":"* `:delay` (`t:non_neg_integer/0`) - Time to wait between keydown and keyup in milliseconds. The default value is `0`.","ref":"PhoenixTest.Playwright.html#press/4-options"},{"type":"function","title":"Examples - PhoenixTest.Playwright.press/4","doc":"|> press(\"#id\", \"Control+Shift+T\")\n    |> press(Selector.button(\"Submit\", exact: true), \"Enter\")","ref":"PhoenixTest.Playwright.html#press/4-examples"},{"type":"function","title":"PhoenixTest.Playwright.screenshot/3","doc":"Takes a screenshot of the current page and saves it to the given file path.\n\nThe file type will be inferred from the file extension on the path you provide.\nThe file is saved in `:screenshot_dir`, see `PhoenixTest.Playwright.Config`.","ref":"PhoenixTest.Playwright.html#screenshot/3"},{"type":"function","title":"Options - PhoenixTest.Playwright.screenshot/3","doc":"* `:full_page` (`t:boolean/0`) - The default value is `true`.\n\n* `:omit_background` (`t:boolean/0`) - Only applicable to .png images. The default value is `false`.","ref":"PhoenixTest.Playwright.html#screenshot/3-options"},{"type":"function","title":"Examples - PhoenixTest.Playwright.screenshot/3","doc":"|> screenshot(\"my-screenshot.png\")\n    |> screenshot(\"my-test/my-screenshot.jpg\")","ref":"PhoenixTest.Playwright.html#screenshot/3-examples"},{"type":"type","title":"PhoenixTest.Playwright.selector/0","doc":"","ref":"PhoenixTest.Playwright.html#t:selector/0"},{"type":"opaque","title":"PhoenixTest.Playwright.t/0","doc":"","ref":"PhoenixTest.Playwright.html#t:t/0"},{"type":"function","title":"PhoenixTest.Playwright.type/4","doc":"Focuses the matching element and simulates user typing.\n\nIn most cases, you should use `PhoenixTest.fill_in/4` instead.","ref":"PhoenixTest.Playwright.html#type/4"},{"type":"function","title":"Options - PhoenixTest.Playwright.type/4","doc":"* `:delay` (`t:non_neg_integer/0`) - Time to wait between key presses in milliseconds. The default value is `0`.","ref":"PhoenixTest.Playwright.html#type/4-options"},{"type":"function","title":"Examples - PhoenixTest.Playwright.type/4","doc":"|> type(\"#id\", \"some text\")\n    |> type(Selector.role(\"heading\", \"Untitled\", exact: true), \"New title\")","ref":"PhoenixTest.Playwright.html#type/4-examples"},{"type":"function","title":"PhoenixTest.Playwright.unwrap/2","doc":"See `PhoenixTest.unwrap/2`.\n\nInvokes `fun` with various Playwright IDs.\nThese can be used to interact with the Playwright\n[`BrowserContext`](`PhoenixTest.Playwright.BrowserContext`),\n[`Page`](`PhoenixTest.Playwright.Page`) and\n[`Frame`](`PhoenixTest.Playwright.Frame`).","ref":"PhoenixTest.Playwright.html#unwrap/2"},{"type":"function","title":"Examples - PhoenixTest.Playwright.unwrap/2","doc":"|> unwrap(&Frame.evaluate(&1.frame_id, \"console.log('Hey')\"))","ref":"PhoenixTest.Playwright.html#unwrap/2-examples"},{"type":"module","title":"PhoenixTest.Playwright.Browser","doc":"Interact with a Playwright `Browser`.\n\nThere is no official documentation, since this is considered Playwright internal.\n\nReferences:\n- https://github.com/microsoft/playwright/blob/main/packages/playwright-core/src/client/browser.ts","ref":"PhoenixTest.Playwright.Browser.html"},{"type":"function","title":"PhoenixTest.Playwright.Browser.new_context/2","doc":"Start a new browser context and return its `guid`.","ref":"PhoenixTest.Playwright.Browser.html#new_context/2"},{"type":"module","title":"PhoenixTest.Playwright.BrowserContext","doc":"Interact with a Playwright `BrowserContext`.\n\nThere is no official documentation, since this is considered Playwright internal.\n\nReferences:\n- https://github.com/microsoft/playwright/blob/main/packages/playwright-core/src/client/browserContext.ts","ref":"PhoenixTest.Playwright.BrowserContext.html"},{"type":"function","title":"PhoenixTest.Playwright.BrowserContext.clear_cookies/2","doc":"Removes all cookies from the context","ref":"PhoenixTest.Playwright.BrowserContext.html#clear_cookies/2"},{"type":"function","title":"PhoenixTest.Playwright.BrowserContext.new_page/1","doc":"Open a new browser page and return its `guid`.","ref":"PhoenixTest.Playwright.BrowserContext.html#new_page/1"},{"type":"function","title":"PhoenixTest.Playwright.BrowserContext.start_tracing/2","doc":"Start tracing. The results can be retrieved via `stop_tracing/2`.","ref":"PhoenixTest.Playwright.BrowserContext.html#start_tracing/2"},{"type":"function","title":"PhoenixTest.Playwright.BrowserContext.stop_tracing/2","doc":"Stop tracing and write zip file to specified output path.\n\nTrace can be viewed via either\n- `npx playwright show-trace trace.zip`\n- https://trace.playwright.dev","ref":"PhoenixTest.Playwright.BrowserContext.html#stop_tracing/2"},{"type":"module","title":"PhoenixTest.Playwright.Case","doc":"ExUnit case module to assist with browser based tests.\n\nSee `PhoenixTest.Playwright` and `PhoenixTest.Playwright.Config`\nfor more information.","ref":"PhoenixTest.Playwright.Case.html"},{"type":"module","title":"PhoenixTest.Playwright.Config","doc":"Configuration options for the Playwright driver.\n\nMost should be set globally in `config/tests.exs`.\nSome can be overridden per test.\n\nAll options:\n* `:browser` (`:android | :chromium | :electron | :firefox | :webkit`) - The default value is `:chromium`.\n\n* `:cli` (`t:String.t/0`) - The default value is `\"assets/node_modules/playwright/cli.js\"`.\n\n* `:headless` (`t:boolean/0`) - The default value is `true`.\n\n* `:js_logger` (`:default | false | (msg -> nil)`) - The default value is `:default`.\n\n* `:screenshot` (`boolean/0 | Keyword.t/0`) - Either a boolean or a keyword list:\n\n    * `:full_page` (`t:boolean/0`) - The default value is `true`.\n\n    * `:omit_background` (`t:boolean/0`) - The default value is `false`.\n\n  The default value is `false`.\n\n* `:screenshot_dir` (`t:String.t/0`) - The default value is `\"screenshots\"`.\n\n* `:browser_launch_timeout` (`t:non_neg_integer/0`) - The default value is `4000`.\n\n* `:timeout` (`t:non_neg_integer/0`) - The default value is `2000`.\n\n* `:slow_mo` (`t:non_neg_integer/0`) - The default value is `0`.\n\n* `:trace` (`boolean/0 | :open`) - The default value is `false`.\n\n* `:trace_dir` (`t:String.t/0`) - The default value is `\"traces\"`.\n\n\n\nOptions that be overridden per test module via the `use PhoenixTest.Playwright.Case` opts:\n- `:browser`\n- `:browser_launch_timeout`\n- `:headless`\n- `:slow_mo`\n\nOptions that be overridden per test via ExUnit `@tag`:\n- `:screenshot`\n- `:trace`","ref":"PhoenixTest.Playwright.Config.html"},{"type":"function","title":"PhoenixTest.Playwright.Config.global/0","doc":"","ref":"PhoenixTest.Playwright.Config.html#global/0"},{"type":"function","title":"PhoenixTest.Playwright.Config.global/1","doc":"","ref":"PhoenixTest.Playwright.Config.html#global/1"},{"type":"function","title":"PhoenixTest.Playwright.Config.setup_all_keys/0","doc":"","ref":"PhoenixTest.Playwright.Config.html#setup_all_keys/0"},{"type":"function","title":"PhoenixTest.Playwright.Config.setup_keys/0","doc":"","ref":"PhoenixTest.Playwright.Config.html#setup_keys/0"},{"type":"function","title":"PhoenixTest.Playwright.Config.validate!/1","doc":"","ref":"PhoenixTest.Playwright.Config.html#validate!/1"},{"type":"module","title":"PhoenixTest.Playwright.Connection","doc":"Stateful, `GenServer` based connection to a Playwright node.js server.\nThe connection is established via `PhoenixTest.Playwright.Port`.\n\nThe state is ever-growing and  never pruned.\nSince we're dealing with (short-lived) tets, that is considered good enough for now.\n\nThe state includes\n- a tree of Playwright artifact guids,\n- all received messages,\n- sent message for which we're expecting a response.\n\nYou won't usually have to use this module directly.\n`PhoenixTest.Playwright` uses this under the hood.","ref":"PhoenixTest.Playwright.Connection.html"},{"type":"function","title":"PhoenixTest.Playwright.Connection.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"PhoenixTest.Playwright.Connection.html#child_spec/1"},{"type":"function","title":"PhoenixTest.Playwright.Connection.ensure_started/0","doc":"Lazy launch. Only start the playwright server if actually needed by a test.","ref":"PhoenixTest.Playwright.Connection.html#ensure_started/0"},{"type":"function","title":"PhoenixTest.Playwright.Connection.initializer/1","doc":"Get the initializer data for a channel.","ref":"PhoenixTest.Playwright.Connection.html#initializer/1"},{"type":"function","title":"PhoenixTest.Playwright.Connection.launch_browser/2","doc":"Launch a browser and return its `guid`.","ref":"PhoenixTest.Playwright.Connection.html#launch_browser/2"},{"type":"function","title":"PhoenixTest.Playwright.Connection.post/2","doc":"Post a message and await the response.\nWe wait for an additional grace period after the timeout that we pass to playwright.","ref":"PhoenixTest.Playwright.Connection.html#post/2"},{"type":"function","title":"PhoenixTest.Playwright.Connection.received/1","doc":"Get all past received messages for a playwright `guid` (e.g. a `PhoenixTest.Playwright.Frame`).","ref":"PhoenixTest.Playwright.Connection.html#received/1"},{"type":"function","title":"PhoenixTest.Playwright.Connection.start_link/0","doc":"","ref":"PhoenixTest.Playwright.Connection.html#start_link/0"},{"type":"function","title":"PhoenixTest.Playwright.Connection.subscribe/2","doc":"Subscribe to messages for a guid and its descendants.","ref":"PhoenixTest.Playwright.Connection.html#subscribe/2"},{"type":"module","title":"PhoenixTest.Playwright.CookieArgs","doc":"Functions to assist with adding cookies to the browser context\n\nA cookie's value must be a binary unless the cookie is signed/encrypted","ref":"PhoenixTest.Playwright.CookieArgs.html"},{"type":"module","title":"Cookie fields - PhoenixTest.Playwright.CookieArgs","doc":"| key          | type        | description |\n| -----------  | ----------- | ----------- |\n| `:name`      | `binary()`  | |\n| `:value`     | `binary()`  | |\n| `:url`       | `binary()`  | *(optional)* either url or domain / path are required |\n| `:domain`    | `binary()`  | *(optional)* either url or domain / path are required |\n| `:path`      | `binary()`  | *(optional)* either url or domain / path are required |\n| `:max_age`   | `float()`   | *(optional)* The cookie max age, in seconds. |\n| `:http_only` | `boolean()` | *(optional)* |\n| `:secure`    | `boolean()` | *(optional)* |\n| `:encrypt`   | `boolean()` | *(optional)* |\n| `:sign`      | `boolean()` | *(optional)* |\n| `:same_site` | `binary()`  | *(optional)* one of \"Strict\", \"Lax\", \"None\" |\n\nTwo of the cookie fields mean nothing to Playwright. These are:\n\n1. `:encrypt`\n2. `:sign`\n\nThe `:max_age` cookie field means the same thing as documented in `Plug.Conn.put_resp_cookie/4`.\nThe `:max_age` value is used to infer the correct `expires` value that Playwright requires.\n\nSee https://playwright.dev/docs/api/class-browsercontext#browser-context-add-cookies","ref":"PhoenixTest.Playwright.CookieArgs.html#module-cookie-fields"},{"type":"type","title":"PhoenixTest.Playwright.CookieArgs.cookie/0","doc":"","ref":"PhoenixTest.Playwright.CookieArgs.html#t:cookie/0"},{"type":"function","title":"PhoenixTest.Playwright.CookieArgs.from_cookie/1","doc":"Converts the cookie kw list into a map suitable for posting","ref":"PhoenixTest.Playwright.CookieArgs.html#from_cookie/1"},{"type":"function","title":"PhoenixTest.Playwright.CookieArgs.from_session_options/2","doc":"Converts the session cookie kw list (with value that is a map) into a map suitable for posting","ref":"PhoenixTest.Playwright.CookieArgs.html#from_session_options/2"},{"type":"type","title":"PhoenixTest.Playwright.CookieArgs.playwright_cookie_args/0","doc":"","ref":"PhoenixTest.Playwright.CookieArgs.html#t:playwright_cookie_args/0"},{"type":"module","title":"PhoenixTest.Playwright.Frame","doc":"Interact with a Playwright `Frame` (usually the \"main\" frame of a browser page).\n\nThere is no official documentation, since this is considered Playwright internal.\n\nReferences:\n- https://github.com/microsoft/playwright/blob/main/packages/playwright-core/src/client/frame.ts","ref":"PhoenixTest.Playwright.Frame.html"},{"type":"function","title":"PhoenixTest.Playwright.Frame.check/3","doc":"","ref":"PhoenixTest.Playwright.Frame.html#check/3"},{"type":"function","title":"PhoenixTest.Playwright.Frame.click/3","doc":"","ref":"PhoenixTest.Playwright.Frame.html#click/3"},{"type":"function","title":"PhoenixTest.Playwright.Frame.content/1","doc":"","ref":"PhoenixTest.Playwright.Frame.html#content/1"},{"type":"function","title":"PhoenixTest.Playwright.Frame.evaluate/3","doc":"","ref":"PhoenixTest.Playwright.Frame.html#evaluate/3"},{"type":"function","title":"PhoenixTest.Playwright.Frame.expect/2","doc":"","ref":"PhoenixTest.Playwright.Frame.html#expect/2"},{"type":"function","title":"PhoenixTest.Playwright.Frame.fill/4","doc":"","ref":"PhoenixTest.Playwright.Frame.html#fill/4"},{"type":"function","title":"PhoenixTest.Playwright.Frame.goto/2","doc":"","ref":"PhoenixTest.Playwright.Frame.html#goto/2"},{"type":"function","title":"PhoenixTest.Playwright.Frame.inner_html/2","doc":"","ref":"PhoenixTest.Playwright.Frame.html#inner_html/2"},{"type":"function","title":"PhoenixTest.Playwright.Frame.press/4","doc":"","ref":"PhoenixTest.Playwright.Frame.html#press/4"},{"type":"function","title":"PhoenixTest.Playwright.Frame.select_option/4","doc":"","ref":"PhoenixTest.Playwright.Frame.html#select_option/4"},{"type":"function","title":"PhoenixTest.Playwright.Frame.set_input_files/4","doc":"","ref":"PhoenixTest.Playwright.Frame.html#set_input_files/4"},{"type":"function","title":"PhoenixTest.Playwright.Frame.title/1","doc":"","ref":"PhoenixTest.Playwright.Frame.html#title/1"},{"type":"function","title":"PhoenixTest.Playwright.Frame.type/4","doc":"","ref":"PhoenixTest.Playwright.Frame.html#type/4"},{"type":"function","title":"PhoenixTest.Playwright.Frame.uncheck/3","doc":"","ref":"PhoenixTest.Playwright.Frame.html#uncheck/3"},{"type":"function","title":"PhoenixTest.Playwright.Frame.url/1","doc":"","ref":"PhoenixTest.Playwright.Frame.html#url/1"},{"type":"function","title":"PhoenixTest.Playwright.Frame.wait_for_selector/2","doc":"","ref":"PhoenixTest.Playwright.Frame.html#wait_for_selector/2"},{"type":"module","title":"PhoenixTest.Playwright.Page","doc":"Interact with a Playwright `Page`.\n\nThere is no official documentation, since this is considered Playwright internal.\n\nReferences:\n- https://github.com/microsoft/playwright/blob/main/packages/playwright-core/src/client/page.ts","ref":"PhoenixTest.Playwright.Page.html"},{"type":"function","title":"PhoenixTest.Playwright.Page.screenshot/2","doc":"","ref":"PhoenixTest.Playwright.Page.html#screenshot/2"},{"type":"function","title":"PhoenixTest.Playwright.Page.update_subscription/2","doc":"","ref":"PhoenixTest.Playwright.Page.html#update_subscription/2"},{"type":"module","title":"PhoenixTest.Playwright.Port","doc":"Start a Playwright node.js server and communicate with it via a `Port`.\n\nA single `Port` response can contain multiple Playwright messages and/or a fraction of a message.\nThe remaining fraction is stored in `buffer` and continiued in the next `Port` response.","ref":"PhoenixTest.Playwright.Port.html"},{"type":"function","title":"PhoenixTest.Playwright.Port.cli_path/0","doc":"","ref":"PhoenixTest.Playwright.Port.html#cli_path/0"},{"type":"function","title":"PhoenixTest.Playwright.Port.from_json/1","doc":"","ref":"PhoenixTest.Playwright.Port.html#from_json/1"},{"type":"function","title":"PhoenixTest.Playwright.Port.open/0","doc":"","ref":"PhoenixTest.Playwright.Port.html#open/0"},{"type":"function","title":"PhoenixTest.Playwright.Port.parse/2","doc":"","ref":"PhoenixTest.Playwright.Port.html#parse/2"},{"type":"function","title":"PhoenixTest.Playwright.Port.post/2","doc":"","ref":"PhoenixTest.Playwright.Port.html#post/2"},{"type":"function","title":"PhoenixTest.Playwright.Port.to_json/1","doc":"","ref":"PhoenixTest.Playwright.Port.html#to_json/1"},{"type":"module","title":"PhoenixTest.Playwright.Selector","doc":"Playright supports different types of locators: CSS, XPath, internal.\n\nThey can mixed and matched by chaining the together.\n\nAlso, you can register [custom selector engines](https://playwright.dev/docs/extensibility#custom-selector-engines)\nthat run right in the browser (Javascript).\n\nThere is no official documentation, since this is considered Playwright internal.\n\nReferences:\n- https://playwright.dev/docs/other-locators\n- https://github.com/microsoft/playwright/blob/main/packages/playwright-core/src/client/locator.ts\n- https://github.com/microsoft/playwright/blob/main/packages/playwright-core/src/utils/isomorphic/locatorUtils.ts","ref":"PhoenixTest.Playwright.Selector.html"},{"type":"function","title":"PhoenixTest.Playwright.Selector.and/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#and/2"},{"type":"function","title":"PhoenixTest.Playwright.Selector.at/1","doc":"","ref":"PhoenixTest.Playwright.Selector.html#at/1"},{"type":"function","title":"PhoenixTest.Playwright.Selector.build/1","doc":"","ref":"PhoenixTest.Playwright.Selector.html#build/1"},{"type":"function","title":"PhoenixTest.Playwright.Selector.button/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#button/2"},{"type":"function","title":"PhoenixTest.Playwright.Selector.concat/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#concat/2"},{"type":"function","title":"PhoenixTest.Playwright.Selector.css/1","doc":"","ref":"PhoenixTest.Playwright.Selector.html#css/1"},{"type":"function","title":"PhoenixTest.Playwright.Selector.has/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#has/2"},{"type":"function","title":"PhoenixTest.Playwright.Selector.label/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#label/2"},{"type":"function","title":"PhoenixTest.Playwright.Selector.link/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#link/2"},{"type":"function","title":"PhoenixTest.Playwright.Selector.menuitem/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#menuitem/2"},{"type":"function","title":"PhoenixTest.Playwright.Selector.none/0","doc":"","ref":"PhoenixTest.Playwright.Selector.html#none/0"},{"type":"function","title":"PhoenixTest.Playwright.Selector.role/3","doc":"","ref":"PhoenixTest.Playwright.Selector.html#role/3"},{"type":"function","title":"PhoenixTest.Playwright.Selector.text/2","doc":"","ref":"PhoenixTest.Playwright.Selector.html#text/2"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.1.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"","ref":"changelog.html#unreleased"},{"type":"extras","title":"Added - Changelog","doc":"- Add locale to avoid console warnings. Commit [becf5e6] @peaceful-james\n\n## [0.6.2] 2025-04-24","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `Frame.evaluate/3`: Don't transform map keys in return value. Example: `js {camelCase: {a: 1}} -> ex %{\"camelCase\": %{\"a\": 1}}`. Previously attempted to underscore and atom-ize keys, which led to issue [#24](https://github.com/ftes/phoenix_test_playwright/pull/24). Commit [5ff530]\n\n## [0.6.1] 2025-04-02","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Support relevant `phoenix_test 0.6` features\n  - ✅ Deprecate `select` with `:from` in favor of `:option` (handled by `PhoenixTest`)\n  - ✅ Allow nesting of `within/3`\n  - ✅ Allow calling `visit/2` sequentially (was already supported)\n  - ❌ Raise error when visiting a defined route: In a feature test, we assert on the rendered page, not the status code.\n\n## [0.6.0] 2025-03-18","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Add and remove cookies: `add_cookies/2`, `add_session_cookie/3`, `clear_cookies/{1,2}` @peaceful-james\n- Add option `browser_launch_timeout` for more fine-grained control (is typically a lot slower than other actions) @s3cur3\n\n## [0.5.0] 2025-02-14","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Docs: Document and validate global and per-test configuration\n- Docs: Document non-standard driver functions (`click/4`, `click_button/4` etc.). Also, exclude standard driver functions from docs.\n- Config: Override config via `Case` opts, e.g. `use PhoenixTest.Playwright.Case, headless: false`\n- Keyboard simulation: `type/{3,4}` and `press/{3,4}`","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Renamed: `PheonixTest.Case` to `PhoenixTest.Playwright.Case`\n  ```diff\n     defmodule MyTest do\n  -    use PhoenixTest.Case, async: true\n  +    use PhoenixTest.Playwright.Case, async: true\n  -    @moduletag :playwright\n  ```\n\n## [0.4.0] 2025-02-03","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Screenshots: `screenshot/{2,3}` function and `screenshot: true` config for auto-capture @s3cur3","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Config: flattened list (remove nested `browser` config), override via top-level ExUnit `@tag ...`s (remove nested `@tag playwright: [...]`)\n  ```diff\n     # config/test.exs\n     config :phoenix_test,\n       playwright: [\n  -      browser: [browser: :chromium, headless: false, slow_mo: 0]\n  +      browser: :chromium,\n  +      headless: false,\n  +      slow_mo: 0\n  ```\n\n## [0.3.0] 2025-01-26","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Auto-convert case of playwright messages keys (snake_case to camelCase)\n\n## [0.2.1] 2025-01-17","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add more config options (browser, JS console) @s3cur3","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Improve error messages @s3cur3\n- Improve setup and docs for contributors @s3cur3\n\n## [0.2.0] 2025-01-09","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- support `phoenix_test@0.5`, `elixir@1.18`, `phoenix_live_view@1.0`\n\n## [0.1.5] 2024-12-15","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- `@tag trace: :open` to auto open recorded Playwright trace in viewer\n\n[5ff530]: https://github.com/ftes/phoenix_test_playwright/commit/5ff530\n[becf5e]: https://github.com/ftes/phoenix_test_playwright/commit/becf5e","ref":"changelog.html#added"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}